# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

from datetime import date
from django.utils.translation import ugettext as _
from basesoft.util.virtual_date import today
from basesoft.web.contrib.order.models import Currency
from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.core.util.controllers import transaction_decorator
from sqlalchemy.sql.expression import and_, func
from we.core.controllers import AbstractWeController
from we.declaration.models import WeDeclaration, WeDeclarationLine
from we.member.models import WeLpp
from we.order.models import WeDeclarationOrder
from we.product.controllers import WeProductController
from we.product.models import WeHqppProduct, WeTbsProductLevel, WeHqppRegulationPeriodProduct


class WeDeclarationController(AbstractWeController):

    @transaction_decorator
    def save_or_update_declaration(self, declaration_id, state):
        self.assert_permissions('we_manage_declaration')

        lines = []
        state_ids = []
        total_fee = 0
        total_weight = 0
        total_quantity = 0
        lpp = self.get_local_producer_partner(state['lpp_id'])

        declaration = self.sa_session.query(WeDeclaration).filter(and_(WeDeclaration.lpp == lpp,
                                                                       WeDeclaration.year == int(state['year']),
                                                                       WeDeclaration.month == int(state['month']))).first()
        if declaration_id is None and declaration is not None:
            months = self.get_months_map()

            if declaration.export_date is not None:
                raise PublicUserException(_('A declaration for year %s and month %s has already been exported '
                                            'to the TBS.' % (state['year'], months[int(state['month']) - 1])))
            raise PublicUserException(_('A declaration for year %s and month %s already '
                                        'exists.' % (state['year'], months[int(state['month']) - 1])))

        # get state line ids
        for line in state['lines']:
            if line['line_id'] in [None, '']:
                continue
            state_ids.append(int(line['line_id']))

        # get declaration
        if declaration_id is None:
            declaration = WeDeclaration()
            declaration.lpp = lpp
            declaration.year = int(state['year'])
            declaration.month = int(state['month'])

            self.logger.info('Saving new declaration')
        else:
            declaration = self.get_declaration(declaration_id)
            self.logger.info('Updating declaration %s' % declaration)

        wpc = WeProductController(self.sa_session, account_id=self.get_account().id)
        period = date(declaration.year, declaration.month, 1)
        regulation_period = wpc.get_regulation_periods(lpp.tbs.id, start_date=period)

        # number of state lines == number of db lines - nothing to delete
        nr_state_lines = len(state_ids)
        if 0 < nr_state_lines < len(declaration.lines):
            _where = """ id NOT IN :state_ids;"""
            if len(state_ids) == 1:
                state_ids = state_ids[0]
                _where = """ id != :state_ids;"""
            # delete lines which are not in the state
            self.logger.info(' - Deleting obsolete lines from declaration')
            self.sa_session.execute("""
                                DELETE FROM tblWeDeclarationLine WHERE declaration_id = :declaration_id AND
                                """ + _where, {
                                    'declaration_id': declaration_id,
                                    'state_ids': state_ids,
                                })
            self.sa_session.expire(declaration)

        # check for new lines and create or update and calculate totals
        for i in xrange(0, len(state['lines'])):
            line = state['lines'][i]
            product = self.sa_session.query(WeHqppProduct).filter(and_(WeHqppProduct.hqpp == lpp.hqpp,
                                                                       WeHqppProduct.sku == line['sku'])).one()
            # get the link between hqpp product and tbs category
            if line['hqpp_period_product_id'] not in [None, '']:
                hqpp_period_product = self.sa_session.query(WeHqppRegulationPeriodProduct).filter(
                    and_(
                        WeHqppRegulationPeriodProduct.id == line['hqpp_period_product_id'],
                        WeHqppRegulationPeriodProduct.tbs_regulation_period == regulation_period
                    )
                ).one()
            else:
                raise PublicUserException(_("Product with SKU %s is not linked to a "
                                            "TBS local category in regulation period %s." % (line['sku'], regulation_period.title)))

            fee = hqpp_period_product.tbs_regulation_period_product.fee
            invoice_basis = hqpp_period_product.tbs_regulation_period_product.tbs_product_level.invoice_basis

            if line['line_id'] in ['', None]:
                new_line = WeDeclarationLine()
            else:
                new_line = self.sa_session.query(WeDeclarationLine).filter(
                    and_(
                        WeDeclarationLine.declaration_id == declaration_id,
                        WeDeclarationLine.id == int(line['line_id'])
                    )
                ).one()

            # calculate line totals and declaration totals
            line['product_total_weight'] = product.get_total_weight_per_unit_for_regulation_period(regulation_period) * line['quantity']

            if invoice_basis == WeTbsProductLevel.INVOICE_BASED_ON_WEIGHT:
                line['amount'] = line['product_total_weight'] * fee
            if invoice_basis == WeTbsProductLevel.INVOICE_BASED_ON_QUANTITY:
                line['amount'] = line['quantity'] * fee

            total_fee += line['amount']
            total_weight += line['product_total_weight']
            total_quantity += line['quantity']

            # save new line
            new_line.hqpp_period_product = hqpp_period_product
            new_line.position = i + 1
            new_line.quantity = line['quantity']
            new_line.product_total_weight = line['product_total_weight']
            new_line.product_total_fee = line['amount']
            lines.append(new_line)

        declaration.total_weight = total_weight
        declaration.total_fee = total_fee
        declaration.total_quantity = total_quantity
        declaration.currency_id = self.sa_session.query(func.iqGetCurrencyIdForCountry(
            lpp.tbs.company.addresses[0].address.country.id,
            today()
        )).one()[0]
        declaration.status = WeDeclaration.STATUS_COMPLETE
        declaration.lines = lines
        self.sa_session.add(declaration)

        self.logger.info(' - Saved declaration %s' % declaration)

        return declaration

    def load_declaration(self, declaration_id):
        self.assert_permissions('we_view_declaration')

        declaration = self.get_declaration(declaration_id)

        currency = self.sa_session.query(Currency).filter(Currency.id == declaration.currency_id).first()

        symbol = currency.symbol
        if not symbol:
            symbol = currency.code

        state = {
            'lpp_id': declaration.lpp_id,
            'lpp_name': declaration.lpp.name,
            'tbs_name': declaration.lpp.tbs.name,
            'year': declaration.year,
            'month': declaration.month,
            'declaration_quantity': declaration.total_quantity,
            'declaration_weight': declaration.total_weight,
            'declaration_fee': declaration.total_fee,
            'currency_symbol': symbol,
            'lines': []
        }

        for line in declaration.lines:
            hqpp_period_product = line.hqpp_period_product
            rpp = hqpp_period_product.tbs_regulation_period_product
            # get hqpp product
            product = self.sa_session.query(WeHqppProduct).filter(WeHqppProduct.id == hqpp_period_product.product_id).one()

            weee_pl = None
            if product.cn8_code.weee_product_level:
                weee_pl = product.cn8_code.weee_product_level.label

            state['lines'].append({
                    'line_id': line.id,
                    'hqpp_period_product_id': line.hqpp_period_product_id,
                    'sku': product.sku,
                    'quantity': line.quantity,
                    'product_total_weight': line.product_total_weight,
                    'amount': line.product_total_fee,
                    'product_name': product.name,
                    'price_per_unit': rpp.fee,
                    'weight_per_unit': product.weight_per_unit,
                    'weight_calculation': product.get_total_weight_per_unit_for_regulation_period(line.hqpp_period_product.tbs_regulation_period),
                    'cn8': product.cn8_code.code,
                    'weee_product_level': weee_pl,
                    'tbs_product_level': rpp.tbs_product_level.label,
                    'invoice_basis': rpp.tbs_product_level.invoice_basis,
                    'is_b2b': hqpp_period_product.is_b2b
            })

        return state

    def get_declaration_id_for_period(self, lpp_id, year, month):
        self.assert_permissions('we_view_declaration')

        lpp = self.get_local_producer_partner(int(lpp_id))
        declaration = self.sa_session.query(WeDeclaration).filter(and_(WeDeclaration.lpp == lpp,
                                                                       WeDeclaration.year == int(year),
                                                                       WeDeclaration.month == int(month))).first()

        if declaration is None:
            return None

        return declaration.id

    def get_declaration(self, declaration_id):

        declaration = self.sa_session.query(WeDeclaration).filter(WeDeclaration.id == declaration_id).one()

        self.assert_local_producer_partner(declaration.lpp_id)

        return declaration

    def get_saved_declaration_periods(self, lpp_id, year):
        d = self.sa_session.query(WeDeclaration).filter(and_(WeDeclaration.lpp_id == int(lpp_id), WeDeclaration.year == year)).all()
        return [r.month for r in d]

    def import_declaration(self, tmp_file, importer, file_path, hqpp_id=None):
        """
        Use importer to insert/update HQPP declarations from a file.
        """

        row_count = importer.get_row_count(file_path)
        if row_count > importer.MAX_RECORDS_FOR_INSTANT_IMPORT:
            importer.import_file(tmp_file, None, hqpp_id=hqpp_id,
                                 row_count=row_count, account_id=self.account_id)
            self.start_task('we.declaration.schedule.WeDeclarationImportTask', {})
        else:
            importer.import_file(tmp_file, self, hqpp_id=hqpp_id,
                                 row_count=row_count, account_id=self.account_id)
        return False

    def validate_declaration(self, importer, file_path, hqpp_id):
        missing_data = []
        sku_list = []
        row_i = 0
        hqpp_id = hqpp_id if hqpp_id not in [None, ''] else None
        hqpp = self.get_member_licenses(hqpp_id).pop()

        for r in importer.read_row(file_path):
            if row_i == 0:
                row_i += 1
                continue
            lpp_id = r[0]
            year = r[1]
            month = r[2]
            sku = r[3]
            is_b2b = None
            if len(r) > 5:
                is_b2b = r[5]
            if is_b2b in [None, '']:
                is_b2b = WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE
            elif is_b2b.lower() == 'b2b':
                is_b2b = WeHqppRegulationPeriodProduct.IS_B2B_TRUE
            else:
                is_b2b = WeHqppRegulationPeriodProduct.IS_B2B_FALSE

            lpp = self.get_local_producer_partner(lpp_id)
            if lpp is None:
                raise PublicUserException(_("No access to local producer partner with ID %s." % lpp_id))
            assert hqpp == lpp.hqpp

            # check for existing declaration
            declaration = self.sa_session.query(WeDeclaration).filter(
                and_(
                    WeDeclaration.lpp_id == lpp_id,
                    WeDeclaration.year == year,
                    WeDeclaration.month == month
                )
            ).first()
            if declaration is not None and declaration.export_date is not None:
                # TODO: remove this after resubmit is implemented
                missing_data.append(_("Declaration for year %s and month %s "
                                      "has been exported to the TBS. Cannot update."))
                return missing_data

            # start checking for sku of that hqpp
            product = self.sa_session.query(WeHqppProduct).filter(
                and_(
                    WeHqppProduct.hqpp == hqpp,
                    WeHqppProduct.sku == sku
                )
            ).first()
            if product is None and sku not in sku_list:
                missing_data.append([row_i,
                                    {'sku': sku,
                                     'product_id': None,
                                     'declaration_period': date(year, month, 2)
                                     },
                                     _('Missing product for SKU.'), hqpp.id, date(year, month, 2)])
                sku_list.append(sku)
                continue

            if is_b2b == WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE:
                if lpp.tbs.get_config('we.tbs.require.end.user.distinction') is True:
                    missing_data.append([row_i,
                                         {'sku': sku,
                                          'product_id': None,
                                          'declaration_period': None
                                          },
                                         _("Line %s with SKU %s "
                                           "requires B2B/B2C distinction." % (row_i, sku)),
                                         hqpp.id, date(year, month, 2)]
                    )
                    continue

            wpc = WeProductController(self.sa_session, account_id=self.get_account().id)
            period = date(year, month, 1)
            regulation_period = wpc.get_regulation_periods(lpp.tbs.id, start_date=period)

            product_tbs_category = self.sa_session.query(WeHqppRegulationPeriodProduct).filter(
                and_(
                    WeHqppRegulationPeriodProduct.product == product,
                    WeHqppRegulationPeriodProduct.tbs_regulation_period == regulation_period,
                    WeHqppRegulationPeriodProduct.is_b2b == is_b2b
                )
            ).first()

            if product is not None and product_tbs_category is None:
                missing_data.append([row_i,
                                     {'sku': sku,
                                      'product_id': product.id,
                                      'declaration_period': date(year, month, 2)
                                      },
                                     _('Missing local category '
                                       'for regulation period %s '
                                       'at take-back scheme %s.') % (regulation_period.title, lpp.tbs.name),
                                     hqpp.id]
                 )

            row_i += 1
        return missing_data

    def get_declarations_by_cn8(self, tbs_id, year, month=None, quarter=None, biannual=None):
        self.assert_permissions('soap_login', 'we_view_declaration')

        tbs = self.get_tbs_licenses(tbs_id).pop()

        # get all exported declarations within the provided period for the Take-back scheme.

        if month is not None:
            assert quarter is None
            assert biannual is None
            months = [month]
        elif quarter is not None:
            assert biannual is None
            months = range(((quarter-1)*3)+1, (quarter*3)+1)
        elif biannual is not None:
            months = range(1, 7) if biannual == WeDeclarationOrder.BI_ANNUAL_FIRST_HALF else range(7, 13)
        else:
            months = range(1, 13)

        # per lpp
        lpps = self.sa_session.query(WeLpp).filter(
            and_(
                WeLpp.tbs == tbs,
                WeLpp.status == WeLpp.STATUS_ACTIVE
            )
        )

        rv = []

        for lpp in lpps:
            self.logger.info(' - Appending declarations for %s ' % lpp)
            rv.append({
                'producer_relation_id': lpp.lpp_relation_id,
                'year': year,
                'month': month,
                'quarter': quarter,
                'bi-annual': biannual,
                'lines': self._get_declarations_by_cn8_by_lpp(lpp, year, months)
            })

        return rv

    def _get_declarations_by_cn8_by_lpp(self, lpp, year, months):
        self.logger.info(' - Declarations for %s and months %s' % (year, months))
        declarations = self.sa_session.query(WeDeclaration).filter(
            and_(
                WeDeclaration.lpp == lpp,
                WeDeclaration.month.in_(months),
                WeDeclaration.year == year,
                WeDeclaration.status == WeDeclaration.STATUS_EXPORTED,
            )
        ).all()

        cn8 = {}

        for declaration in declarations:
            for line in declaration.lines:
                if not cn8.has_key(line.hqpp_period_product.product.cn8):
                    cn8[line.hqpp_period_product.product.cn8] = []
                cn8[line.hqpp_period_product.product.cn8].append(line)

        rv = []

        ordered = sorted(cn8.keys())
        for c in ordered:
            rv.append({
                'cn8': c,
                'weight': sum([l.product_total_weight for l in cn8[c]]),
                'quantity': sum([l.quantity for l in cn8[c]])
            })

        return rv
