# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
import os

from datetime import date
from django.utils.translation import ugettext as _
from basesoft.web.contrib.file.controllers import ImportFileController
from basesoft.web.core.importers import Utf8CsvImporter, XlsxImporter
from sqlalchemy.sql.expression import and_
from we.core.importers import WeImportException
from we.declaration.models import WeDeclaration
from we.member.models import WeLpp, WeHqppLicense
from we.product.models import WeHqppRegulationPeriodProduct, WeHqppProduct
from we.tbs.models import WeTbsRegulationPeriod


class WeDeclarationAbstractImporter(ImportFileController):
    IMPORT_FILE_TYPE = 'we.declaration.Declaration'

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """
        base_filename, file_extension = os.path.splitext(tmp_file.file_name)

        importer = None
        if file_extension in ('.csv', '.txt'):
            importer = WeDeclarationCsvImporter

        elif file_extension == '.xlsx':
            importer = WeDeclarationXlsxImporter

        else:
            raise Exception(_('Cannot import for extension: %s') % file_extension)

        return importer


class WeDeclarationImporter(WeDeclarationAbstractImporter):

    MAX_RECORDS_FOR_INSTANT_IMPORT = 0
    NUMBER_EXPECTED_COLUMNS = 6
    STOP_AFTER_NUMBER_EMPTY_ROWS = 1

    import_state = None
    hqpp = None
    lpps = None
    callback = None
    row_number = 1

    def __init__(self, session, **kwargs):
        ImportFileController.__init__(self, session, **kwargs)
        self.import_state = {}
        self.hqpp = {}
        self.lpps = {}
        self.row_number = 1

    @staticmethod
    def get_notification_callback():
        return {
            'js_module': 'we_declaration_manager'
        }

    @staticmethod
    def get_failure_notification_callback():
        return {
            'js_module': 'we_declaration_import_wizard'
        }

    @staticmethod
    def get_notification_subject():
        return None

    @staticmethod
    def get_notification_text():
        return None

    def _import_row(self, row, controller, dryrun=False, hqpp_id=None, row_count=None, **kwargs):

        lpp_id = int(row[0])
        year = int(row[1])
        month = int(row[2])
        sku = row[3]
        quantity = int(row[4])
        is_b2b = row[5] if len(row) == 6 else None

        loaded_state = {}
        last_row_error = False
        self.row_number += 1

        if not self.hqpp.has_key(hqpp_id):
            hqpp = self.sa_session.query(WeHqppLicense).filter(WeHqppLicense.license_id == hqpp_id).first()
            self.hqpp[hqpp_id] = hqpp
        else:
            hqpp = self.hqpp[hqpp_id]

        if hqpp is None:
            raise WeImportException(_('There is no HQPP with ID %s.' % hqpp_id))

        if not self.lpps.has_key(lpp_id):
            lpp = self.sa_session.query(WeLpp).filter(WeLpp.id == lpp_id, WeLpp.hqpp_id == hqpp_id).first()
            self.lpps[lpp_id] = lpp
        else:
            lpp = self.lpps[lpp_id]

        if lpp is None:
            raise WeImportException(_("Producer %s has no access to LPP with ID %s " % (hqpp.name, lpp_id)))

        if is_b2b in [None, '']:
            is_b2b = WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE
        elif is_b2b.lower() == 'b2b':
            is_b2b = WeHqppRegulationPeriodProduct.IS_B2B_TRUE
        else:
            is_b2b = WeHqppRegulationPeriodProduct.IS_B2B_FALSE

        if is_b2b == WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE:
            if lpp.tbs.get_config('we.tbs.require.end.user.distinction') is True:
                if self.row_number == row_count:
                    last_row_error = True
                else:
                    raise WeImportException(_("Line %s with SKU %s "
                                              "requires B2B/B2C distinction." % (self.row_number, sku)))

        if last_row_error is False:
            if not self.import_state.has_key(lpp_id):
                self.import_state[lpp_id] = {}
            if not self.import_state[lpp_id].has_key(year):
                self.import_state[lpp_id][year] = {}
            if not self.import_state[lpp_id][year].has_key(month):
                self.import_state[lpp_id][year][month] = {}

            # add line to the state
            self.import_state[lpp_id][year][month]['lpp_id'] = lpp_id
            self.import_state[lpp_id][year][month]['year'] = year
            self.import_state[lpp_id][year][month]['month'] = month
            if not self.import_state[lpp_id][year][month].has_key('lines'):
                self.import_state[lpp_id][year][month]['lines'] = []

            line = {
                'line_id': None,
                'sku': sku,
                'quantity': quantity,
                'is_b2b': is_b2b
            }
            self.import_state[lpp_id][year][month]['lines'].append(line)

        if self.row_number >= row_count:
            for lpp_id in self.import_state.keys():
                for year in self.import_state[lpp_id].keys():
                    for month in self.import_state[lpp_id][year].keys():

                        lpp = self.lpps[lpp_id]
                        declaration = self.sa_session.query(WeDeclaration)\
                            .filter(and_(WeDeclaration.lpp == lpp,
                                         WeDeclaration.year == year,
                                         WeDeclaration.month == month)).first()

                        _filter = and_(WeTbsRegulationPeriod.tbs == lpp.tbs,
                                       WeTbsRegulationPeriod.start_date <= date(year, month, 1))
                        regulation_period = self.sa_session.query(WeTbsRegulationPeriod).filter(_filter)\
                            .order_by(WeTbsRegulationPeriod.start_date.desc()).first()

                        # loop the lines from the import file and compare them to the loaded
                        # else just save a new declaration
                        _id = None
                        if declaration is not None:
                            if declaration.export_date is not None:
                                raise WeImportException(_("This declaration has been exported to the TBS."))
                            loaded_state = controller.load_declaration(declaration.id)
                            _id = declaration.id
                        lines_state = self._create_declaration_state(hqpp, regulation_period,
                                                                     self.import_state[lpp_id][year][month]['lines'],
                                                                     loaded_state)
                        self.import_state[lpp_id][year][month]['lines'] = lines_state
                        controller.save_or_update_declaration(_id, self.import_state[lpp_id][year][month])

            if last_row_error is True:
                raise WeImportException(_("Line %s with SKU %s "
                                          "requires B2B/B2C distinction." % (self.row_number, sku)))

        return True

    def _create_declaration_state(self, hqpp, regulation_period, lines, loaded_state):
        matched = False

        for line in lines:
            sku = line['sku']

            # if sku is in loaded state lines set the line id and the hqpp period product
            # else add new line
            if loaded_state.has_key('lines'):
                for l in loaded_state['lines']:
                    if sku.lower() == l['sku'].lower():
                        line['hqpp_period_product_id'] = l['hqpp_period_product_id']
                        line['line_id'] = l['line_id']
                        matched = True

            if matched is False:
                # get the hqpp product corresponding to the SKU
                product = self.sa_session.query(WeHqppProduct).filter(
                    WeHqppProduct.hqpp == hqpp,
                    WeHqppProduct.sku == sku
                ).one()
                product_tbs_category = self.sa_session.query(WeHqppRegulationPeriodProduct).filter(
                    and_(
                        WeHqppRegulationPeriodProduct.product == product,
                        WeHqppRegulationPeriodProduct.tbs_regulation_period == regulation_period,
                        WeHqppRegulationPeriodProduct.is_b2b == line['is_b2b']
                    )
                ).first()
                line['hqpp_period_product_id'] = product_tbs_category.id
            matched = False
        return lines

class WeDeclarationCsvImporter(WeDeclarationImporter, Utf8CsvImporter):
    """
    Importer for CSV
    """

    def read_row(self, file_path):
        return self._read_row(file_path)


class WeDeclarationXlsxImporter(WeDeclarationImporter, XlsxImporter):
    """
    Importer for XLSX
    """

    def read_row(self, file_path):

        active_sheet = self._read_row(file_path)

        for row in active_sheet.iter_rows():
            values = []
            cell_i = 0
            for cell in row:
                cell_i += 1
                if cell_i > 6:
                    break

                value = cell.internal_value
                if value is None:
                    value = ''
                if type(value) == float:
                    value = int(value)
                values.append(value)

            yield values