# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
import os

from django.utils.translation import ugettext as _
from basesoft.util.virtual_date import today
from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from django.conf import settings
from we.declaration.controllers import WeDeclarationController
from we.declaration.importers import WeDeclarationXlsxImporter, WeDeclarationCsvImporter


class WeDeclarationRpcHandler:

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.wdc = WeDeclarationController(self.sa_session, account_id=self.account_id, request=request)

    def save_or_update_declaration(self, declaration_id, state):
        self.wdc.save_or_update_declaration(declaration_id, state)

    def load_declaration(self, declaration_id):
        return self.wdc.load_declaration(declaration_id)

    def get_declaration_id_for_period(self, lpp_id, year, month):
        return self.wdc.get_declaration_id_for_period(lpp_id, year, month)

    def import_declaration_file(self, file_hash, hqpp_id):
        tmp_file = file_hash[0]
        base_filename, file_extension = self.wdc.get_file_extension(tmp_file.file_name, tmp_file.file_hash)

        if file_extension in ('.csv', '.txt'):
            importer = WeDeclarationCsvImporter

        elif file_extension == '.xlsx':
            importer = WeDeclarationXlsxImporter

        else:
            raise PublicUserException(_('Cannot import for extension: %s' % file_extension))

        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)
        importer = importer(self.sa_session, account_id=self.account_id)

        # validate SKUs
        missing_skus = self.wdc.validate_declaration(importer, file_path, hqpp_id)
        if len(missing_skus) > 0:
            return missing_skus

        return self.wdc.import_declaration(tmp_file, importer, file_path, hqpp_id=hqpp_id)

    def get_available_months(self, lpp_id, year):
        month = today().month
        y = today().year
        rv = []
        months = self.wdc.get_months_map()

        periods = self.wdc.get_saved_declaration_periods(lpp_id, year)
        if int(year) < y:
            month = 12
        for i in xrange(1, month + 1):

            if i in periods:
                continue
            rv.append({
                'value': i,
                'label': months[i - 1]
            })

        return rv

    def get_all_months(self):
        return self.wdc.get_months()

    def get_years(self):
        year = today().year
        rv = []

        for i in xrange(year - 2, year + 1):
            rv.append({
                'value': i,
                'label': i
            })

        return rv

    def validate_declaration_form(self, args):
        form_handler = FormValidator({
            'lpp_id': [(plugins.Required, {})],
            'sku': [(plugins.Required, {})],
            'quantity': [(plugins.Required, {})],
            'year': [(plugins.Required, {})],
            'month': [(plugins.Required, {})],
        })

        return form_handler.validate(args)

handlers = (WeDeclarationRpcHandler, )