'''
Created on Jan 14, 2015

@author: rado
'''

import datetime
import os

from openpyxl.cell import get_column_letter

from decimal import Decimal, InvalidOperation
from django.utils.translation import ugettext as _
from basesoft.web.contrib.file.controllers import ImportFileController, ImportFileException
from sqlalchemy.sql.expression import and_
from basesoft.web.core.importers import Utf8CsvImporter, XlsxImporter
from iqportal.weee.core.models import WEEEProductLevel
from we.member.models import WeHqppLicense, WeLpp
from we.product.models import WeHqppProduct, WeTbsProductLevel, WeUnuCode, WeCN8, \
    WeCN8PeriodProduct, WeTbsRegulationPeriodProduct, WeHqppRegulationPeriodProduct
from we.tbs.models import WeTbsLicense, WeTbsRegulationPeriod


class WeImportException(ImportFileException):
    pass


class CN8AbstractImporter(ImportFileController):
    IMPORT_FILE_TYPE = 'we.product.CN8ToWeeeCategory'

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """
        base_filename, file_extension = os.path.splitext(tmp_file.file_name)

        importer = None
        if file_extension in ('.csv', '.txt'):
            importer = CN8CsvImporter

        elif file_extension == '.xlsx':
            importer = CN8XlsxImporter

        else:
            raise Exception(_('Cannot import for extension: %s') % file_extension)

        return importer

class CN8LineImporter(CN8AbstractImporter):
    MAX_RECORDS_FOR_INSTANT_IMPORT = 10000
    NUMBER_EXPECTED_COLUMNS = 5
    weee_cat_map = None
    unu_code_map = None

    def __init__(self, session, **kwargs):
        ImportFileController.__init__(self, session, **kwargs)
        self.weee_cat_map = {}
        self.unu_code_map = {}

    def _import_row(self, row, controller, dryrun=False, **kwargs):
        try:
            int(row[0])
        except ValueError:
            raise WeImportException(_('Invalid CN8 code (%s) found, must be numerical.' % row[0]))

        cn8 = self.sa_session.query(WeCN8).filter(WeCN8.code == int(row[0])).first()

        if row[2] not in [None, '']:
            # weee category label
            row_weee_cat = row[2]
            if not self.weee_cat_map.has_key(row_weee_cat):
                weee_cat = self.sa_session.query(WEEEProductLevel).filter(WEEEProductLevel.label == row_weee_cat).first()
                if weee_cat is None:
                    raise WeImportException(_('WEEE category with label %s does not exist.' % row_weee_cat))
                self.weee_cat_map[weee_cat.label] = weee_cat.id

            # unu code
            row_unu = row[3].strip()
            row_unu_desc = row[4].strip()
            if row_unu == '':
                self.unu_code_map[row_unu] = None
            elif not self.unu_code_map.has_key(row_unu):
                unu_code = self.sa_session.query(WeUnuCode).filter(WeUnuCode.code == row_unu).first()
                if unu_code is None:
                    unu_code = WeUnuCode()
                    unu_code.code = row_unu
                    unu_code.description = row_unu_desc
                    self.sa_session.add(unu_code)
                    self.sa_session.flush()
                self.unu_code_map[unu_code.code] = unu_code.id

            if cn8 is None:

                sql = """
                      INSERT INTO tblWeCN8(code, unu_code_id, weee_product_level_id, description, created, last_updated) VALUES
                      (:cn8_code, :unu_code_id, :weee_product_id, :description, NOW(), NOW())
                      """
            else:
                sql = """
                        UPDATE tblWeCN8 SET unu_code_id = :unu_code_id, weee_product_level_id = :weee_product_id,
                        description = :description, last_updated= NOW() WHERE code = :cn8_code;
                        """
            values = {
                'unu_code_id': self.unu_code_map[row_unu],
                'weee_product_id': self.weee_cat_map[row_weee_cat],
                'description': row[1],
                'cn8_code': int(row[0])
            }
            self.sa_session.execute(sql, values)
        else:
            # weee category and unu are empty
            if cn8 is None:
                sql = """
                      INSERT INTO tblWeCN8(code, description, created, last_updated) VALUES(:cn8_code, :description, NOW(), NOW())
                      """
            else:
                sql = """
                        UPDATE tblWeCN8 SET description = :description, last_updated = NOW() WHERE code = :cn8_code;
                        """

            values = {
                'description': row[1],
                'cn8_code': int(row[0])
            }
            self.sa_session.execute(sql, values)
        return cn8

    @staticmethod
    def get_notification_text():
        return ''

    @staticmethod
    def get_notification_subject():
        return 'CN8 to TBS local category import'

    @staticmethod
    def get_failure_notification_callback():
        return None

    @staticmethod
    def get_notification_callback():
        return None


class CN8CsvImporter(CN8LineImporter, Utf8CsvImporter):
    """
    Importer for CSV
    """

    def read_row(self, file_path):
        for row in self._read_row(file_path):
            yield self.get_only_expected_columns(row)


class CN8XlsxImporter(CN8LineImporter, XlsxImporter):
    """
    Importer for XLSX
    """

    def read_row(self, file_path):

        active_sheet = self._read_row(file_path)

        for row in active_sheet.iter_rows():
            values = []

            row = self.get_only_expected_columns(row)
            for cell in row:

                value = cell.internal_value
                if value is None:
                    value = ''
                if type(value) == float:
                    value = int(value)
                values.append(value)

            yield values


class HqppProductAbstractImporter(ImportFileController):
    IMPORT_FILE_TYPE = 'we.product.HqppProduct'

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """
        base_filename, file_extension = os.path.splitext(tmp_file.file_name)

        importer = None
        if file_extension in ('.csv', '.txt'):
            importer = HqppProductCsvImporter

        elif file_extension == '.xlsx':
            importer = HqppProductXlsxImporter

        else:
            raise Exception(_('Cannot import for extension: %s') % file_extension)

        return importer


class HqppProductLineImporter(HqppProductAbstractImporter):

    MAX_RECORDS_FOR_INSTANT_IMPORT = 0
    NUMBER_EXPECTED_COLUMNS = None

    first_row = True
    rp_dict = None
    tbs_dict = None
    callback = None

    def __init__(self, session, **kwargs):
        ImportFileController.__init__(self, session, **kwargs)
        self.rp_dict = {}
        self.tbs_dict = {}

    def set_notification_callback(self):
        self.callback = {
            'js_module': 'we_hqpp_product_manager'
        }

    def get_notification_callback(self):
        return self.callback

    @staticmethod
    def get_failure_notification_callback():
        return {
            'js_module': 'we_hqpp_product_sheet_import'
        }

    @staticmethod
    def get_notification_subject():
        return None

    @staticmethod
    def get_notification_text():
        return None

    def _import_row(self, row, controller, dryrun, hqpp_id=None, regulation_periods=None):
        if self.first_row is True:
            self.first_row = False
            self.set_notification_callback()
            return

        hqpp = self.sa_session.query(WeHqppLicense).filter(WeHqppLicense.id==hqpp_id).one()
        product = self.sa_session.query(WeHqppProduct)\
            .filter(and_(WeHqppProduct.sku == row[0],
                         WeHqppProduct.hqpp == hqpp)).first()
        values = {}
        tbs_rpp = []
        is_b2b = WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE

        for i in xrange(len(self._file_headers)):
            # if column has no value skip it
            if row[i] in [None, '']:
                continue

            h = self._file_headers[i].lower()

            if h == 'cn8':
                cn8 = self.sa_session.query(WeCN8).filter(WeCN8.code == row[i]).first()
                if cn8 is None:
                    raise WeImportException(_("CN8 code %s does not exist in the database. "
                                              "Please add it before importing again.") % row[i])

            if h == 'is_b2b':
                is_b2b_val = row[i]
                if is_b2b_val.lower() == 'b2b':
                    is_b2b = WeHqppRegulationPeriodProduct.IS_B2B_TRUE
                if is_b2b_val.lower() == 'b2c':
                    is_b2b = WeHqppRegulationPeriodProduct.IS_B2B_FALSE
                continue

            # if headers starts with tbs- get the number and then tbs object
            if len(h) > 4 and h[:4] == 'tbs-':
                tbs_id = int(h[4:])
                tbs_product_label = row[i]

                if not self.tbs_dict.has_key(h):
                    tbs = self.sa_session.query(WeTbsLicense).filter(WeTbsLicense.license_id == tbs_id).first()
                    if tbs is None:
                        raise WeImportException(_("There is no such TBS with ID %s ") % tbs_id)
                    self.tbs_dict[h] = tbs
                else:
                    tbs = self.tbs_dict[h]

                    # check that HQPP has LPPs for that TBS
                    lpps = []
                    for lpp in hqpp.lpps:
                        if lpp.status == WeLpp.STATUS_ACTIVE and lpp.tbs.id == tbs.id:
                            lpps.append(lpp)
                    if len(lpps) == 0:
                        raise WeImportException(_("Producer %s has no local producers registered "
                                                  "at take-back scheme %s." % (hqpp.name, tbs.name)))

                # check whether the user has selected a regulation period for tbs
                if not regulation_periods.has_key(h):
                    WeImportException(_("A regulation period for take-back scheme %s is not selected. "
                                        "Please select one and import again."))

                # check whether the regulation period exists
                if not self.rp_dict.has_key(h):
                    rp_id = regulation_periods[h]
                    rp = self.sa_session.query(WeTbsRegulationPeriod)\
                        .filter(and_(WeTbsRegulationPeriod.id == rp_id,
                                     WeTbsRegulationPeriod.tbs_id == tbs.id)).first()
                    if rp is None:
                        WeImportException(_("Regulation period with ID %s for take-back scheme %s "
                                            "does not exist. Either select another one or create one."))
                    self.rp_dict[h] = rp
                else:
                    rp = self.rp_dict[h]

                # check whether local category exists
                pl = self.sa_session.query(WeTbsProductLevel)\
                    .filter(WeTbsProductLevel.tbs_id == tbs_id,
                            WeTbsProductLevel.label == tbs_product_label).first()
                if pl is None:
                    raise WeImportException(_("There is no such category with "
                                              "label %s at TBS with ID %s ") % (row[i], tbs_id))

                # check whether local category is part of regulation period
                trpp = self.sa_session.query(WeTbsRegulationPeriodProduct).join(WeTbsProductLevel)\
                    .filter(WeTbsProductLevel.tbs_id == tbs_id,
                            WeTbsProductLevel.label == tbs_product_label,
                            WeTbsRegulationPeriodProduct.tbs_regulation_period == rp).first()
                if trpp is None:
                    raise WeImportException(_("TBS local category %s is not part of regulation period %s. "
                                              "Please make sure all local categories are already "
                                              "in the system and are part of the regulation period "
                                              "before importing.") % (row[i], rp.label))
                tbs_rpp.append(trpp)
                continue

            values[h] = row[i]
            values['created'] = datetime.datetime.now()
            values['last_updated'] = datetime.datetime.now()
            values['hqpp_id'] = hqpp.id

        if product is None:
            args = values.keys()
            vals = [':' + val for val in args]

            args = '(' + ','.join(args) + ')'
            vals = '(' + ','.join(vals) + ')'

            sql = """
                  INSERT INTO tblWeHqppProduct %s
                  VALUES %s
                  """ % (args, vals)
            self.sa_session.execute(sql, values)
            # get the newly saved product and link it to TBS product levels
            product = self.sa_session.query(WeHqppProduct)\
                .filter(and_(WeHqppProduct.sku == values['sku'],
                             WeHqppProduct.hqpp == hqpp)).first()

            for l in tbs_rpp:
                hrpp = WeHqppRegulationPeriodProduct()
                hrpp.product = product
                hrpp.tbs_regulation_period_product = l
                hrpp.tbs_regulation_period = l.tbs_regulation_period
                if is_b2b is not None:
                    hrpp.is_b2b = is_b2b
                self.sa_session.add(hrpp)
            self.sa_session.flush()
            return product

        args = values.keys()
        params = ''
        for k in args:
            if k == 'sku' or k == 'hqpp_id':
                continue
            params += '%s = :%s,' % (k, k)
        # remove the comma at the end
        params = params[:len(params) - 1]

        sql = """
                UPDATE tblWeHqppProduct SET %s WHERE sku = :sku AND hqpp_id = :hqpp_id;
                """ % params
        self.sa_session.execute(sql, values)

        for trpp in tbs_rpp:
            rp = trpp.tbs_regulation_period
            # get the link of tbs regulation product to hqpp product for the given regulation product
            hrpp = self.sa_session.query(WeHqppRegulationPeriodProduct)\
                .filter(and_(WeHqppRegulationPeriodProduct.product_id == product.id,
                             WeHqppRegulationPeriodProduct.is_b2b == is_b2b,
                             WeHqppRegulationPeriodProduct.tbs_regulation_period == rp)).first()

            if hrpp is None:
                # add new one
                hrpp = WeHqppRegulationPeriodProduct()
                hrpp.product = product
                hrpp.tbs_regulation_period_product = trpp
                hrpp.tbs_regulation_period = trpp.tbs_regulation_period
                if is_b2b is not None:
                    hrpp.is_b2b = is_b2b
            else:
                # update
                hrpp.tbs_regulation_period_product = trpp
            self.sa_session.add(hrpp)
        self.sa_session.flush()
        return product


class HqppProductCsvImporter(HqppProductLineImporter, Utf8CsvImporter):
    """
    Importer for CSV
    """
    headers = 0

    def read_row(self, file_path):
        for row in self._read_row(file_path):
            self.headers += 1
            if self.headers == 2:
                self.NUMBER_EXPECTED_COLUMNS = len(row)
                continue
            yield self.get_only_expected_columns(row)

class HqppProductXlsxImporter(HqppProductLineImporter, XlsxImporter):
    """
    Importer for XLSX
    """

    def read_row(self, file_path):

        active_sheet = self._read_row(file_path)
        row_i = 1
        for row in active_sheet.iter_rows():
            values = []
            cell_i = 1
            if row_i == 1:
                self.NUMBER_EXPECTED_COLUMNS = len(row)
                row_i += 1

            row = self.get_only_expected_columns(row)
            for cell in row:
                value = cell.internal_value
                if value is None:
                    value = ''
                # make cn8 an int not float
                if cell_i == 16 and type(value) == float:
                    value = int(value)
                if cell_i > 16 and value != '':
                    if not isinstance(value, unicode):
                        WeImportException("Column '%s' "
                                          "must be formatted to text." % get_column_letter(cell_i))
                values.append(value)
                cell_i += 1

            yield values


class TbsPriceSheetAbstractImporter(ImportFileController):
    IMPORT_FILE_TYPE = 'we.product.TbsPriceSheet'

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """
        base_filename, file_extension = os.path.splitext(tmp_file.file_name)

        importer = None
        if file_extension in ('.csv', '.txt'):
            importer = TbsPriceSheetCsvImporter

        elif file_extension == '.xlsx':
            importer = TbsPriceSheetXlsxImporter

        else:
            raise Exception(_('Cannot import for extension: %s') % file_extension)

        return importer


class TbsPriceSheetLineImporter(TbsPriceSheetAbstractImporter):

    MAX_RECORDS_FOR_INSTANT_IMPORT = 0
    NUMBER_EXPECTED_COLUMNS = 5
    number_of_rows = 1
    local_categories = None
    new_categories = None
    callback = None

    def __init__(self, session, **kwargs):
        ImportFileController.__init__(self, session, **kwargs)
        self.number_of_rows = 1  # reset the number of rows.
        self.local_categories = []
        self.new_categories = []
        self._parents = {}

    def set_notification_callback(self, rp_id):
        self.callback = {
            'js_module': 'we_tbs_regulation_period_compare',
            'parameters': [rp_id]
        }

    def get_notification_callback(self):
        return self.callback

    @staticmethod
    def get_failure_notification_callback():
        return {
            'js_module': 'we_tbs_price_sheet_import'
        }

    @staticmethod
    def get_notification_subject():
        return 'New regulation period'

    @staticmethod
    def get_notification_text():
        return 'Use the perform action button to view the changes compared to the previous period'

    def _import_row(self, row, controller, dryrun, tbs_id=None, regulation_period_id=None,
                    regulation_label=None, regulation_year=None, regulation_month=None, weight_settings=None,
                    row_count=None, **kwargs):
        parent_level = None
        parent_level_id = None
        main_category = None if row[0] in [None, ''] else unicode(row[0]).strip()
        category_label = unicode(row[1]).strip()
        description = unicode(row[2]).strip()
        invoice_basis = row[3]
        fee = row[4]
        if type(fee) in (str, unicode, ):
            fee = fee.strip()
            if fee == '':
                fee = None
        five_places = Decimal(10) ** -5
        self.number_of_rows += 1

        tbs = self.sa_session.query(WeTbsLicense).filter(WeTbsLicense.id == tbs_id).first()

        if tbs is None:
            raise WeImportException(_('There is no TBS in the system with ID %s' % tbs_id))

        if main_category is not None:
            # Get the main category from the locally stored list of main categories.
            if not self._parents.has_key(main_category):
                raise WeImportException(_("The main category was not found in the file preceding this record. "
                                          "Make sure the order of the categories is correct."))
            parent_level = self._parents[main_category]
            parent_level_id = parent_level.id

        product_level = self.sa_session.query(WeTbsProductLevel)\
            .filter(and_(WeTbsProductLevel.tbs == tbs,
                         WeTbsProductLevel.label == category_label,
                         WeTbsProductLevel.parent_level == parent_level)).first()

        if product_level is not None and product_level.description.lower() != description.lower():
            product_level = None
        elif product_level is not None and product_level.archived:  # previously archived product levels should be unarchived in case they are reused.
            controller.logger.info('Product level (%s) unarchived by system due to new import.' % product_level)
            product_level.archived = False
            self.sa_session.add(product_level)

        if fee is not None:
            fee = str(fee)
            try:
                fee = Decimal(fee).quantize(five_places)
            except InvalidOperation:
                raise WeImportException('The fee provided is not a valid decimal number, '
                                        'please remove any currency symbols.')
            if fee.is_signed() and not tbs.get_config('we.tbs.use.negative.fee'):
                raise WeImportException(_("TBS %s does not allow negative fees." % tbs.name))

        if invoice_basis in [None, '']:
            # if parent_level is None current row is a main category
            if parent_level is None:
                raise WeImportException('Missing basis for invoice. Parent does not have one either.')
            invoice_basis = parent_level.invoice_basis
        else:
            if invoice_basis.lower() == 'quantity':
                invoice_basis = WeTbsProductLevel.INVOICE_BASED_ON_QUANTITY
            elif invoice_basis.lower() == 'weight':
                invoice_basis = WeTbsProductLevel.INVOICE_BASED_ON_WEIGHT
            elif invoice_basis.lower() == 'fixed annual fee':
                invoice_basis = WeTbsProductLevel.INVOICE_BASED_ON_FIXED_ANNUAL_FEE
            else:
                raise WeImportException(_("Wrong value '%s' for basis for invoice. "
                                          "Use 'quantity', 'weight' or 'fixed annual fee'." % invoice_basis))

        if fee is None and main_category is not None:
            f = None
            # get the fee from the parent category

            for i in xrange(1, len(self.local_categories) + 1):
                if self.local_categories[-i]['id'] == parent_level.id:
                    f = self.local_categories[-i]['fee']
                    break
            # if parent category is not in the import list then ask user for fee
            if f is None:
                raise WeImportException('Fee should be provided for this category '
                                        'because the main category fee is not listed in the price sheet.')
            fee = f

        category_key = (category_label, parent_level_id)
        if category_key in self.new_categories:
            raise WeImportException(_("You cannot have two categories with the same label '%s' "
                                      "and the same parent category ID '%s' "
                                      "in the same regulation period." % (category_label, parent_level_id)))
        self.new_categories.append(category_key)

        # add new level
        if product_level is None:
            product_level = controller.save_or_update_tbs_category(None, category_label,
                                                                   description, tbs,
                                                                   parent_level_id=parent_level_id,
                                                                   invoice_basis=invoice_basis)
        if parent_level is None:
            self._parents[product_level.label] = product_level

        controller.logger.info('Tbs product level [%s] imported.' % category_label)
        # create dictionary for adding to regulation period
        self.local_categories.append({'id': product_level.id, 'fee': fee})

        if self.number_of_rows >= row_count:
            if regulation_period_id is None:
                regulation_year = int(regulation_year)
                regulation_month = int(regulation_month)
                weight_settings = int(weight_settings)
            else:
                rp = controller.get_regulation_period_by_id(tbs.id, regulation_period_id)
                regulation_year = rp.start_date.year
                regulation_month = rp.start_date.month
                regulation_label = rp.label
                weight_settings = rp.weight_settings

            rp = controller\
                .save_or_update_regulation_period(regulation_period_id, tbs.id, regulation_label,
                                                  regulation_year, regulation_month, weight_settings, self.local_categories)
            controller.logger.info(' - Finished importing tbs product levels '
                                   'for regulation period [%s].' % rp.label)
            self.set_notification_callback(rp.id)

        return product_level


class TbsPriceSheetCsvImporter(TbsPriceSheetLineImporter, Utf8CsvImporter):
    """
    Importer for CSV
    """

    def read_row(self, file_path):
        for row in self._read_row(file_path):
            yield self.get_only_expected_columns(row)


class TbsPriceSheetXlsxImporter(TbsPriceSheetLineImporter, XlsxImporter):
    """
    Importer for XLSX
    """

    def read_row(self, file_path):

        active_sheet = self._read_row(file_path)

        for row in active_sheet.iter_rows():
            values = []
            cell_i = 0
            row = self.get_only_expected_columns(row)
            for cell in row:
                cell_i += 1

                value = cell.internal_value

                if cell_i == 1 and value is not None:
                    if not isinstance(value, unicode):
                        WeImportException("Column 'Main category label' must be formatted to text.")
                if cell_i == 2:
                    if not isinstance(value, unicode):
                        WeImportException("Column 'Category label' must be formatted to text.")

                values.append(value)

            yield values


class CN8ToTbsProductLevelAbstractImporter(ImportFileController):
    IMPORT_FILE_TYPE = 'we.product.CN8ToTbsProductLevel'

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """
        base_filename, file_extension = os.path.splitext(tmp_file.file_name)

        importer = None
        if file_extension in ('.csv', '.txt'):
            importer = CN8ToTbsProductLevelCsvImporter

        elif file_extension == '.xlsx':
            importer = CN8ToTbsProductLevelXlsxImporter

        else:
            raise Exception(_('Cannot import for extension: %s') % file_extension)

        return importer


class CN8ToTbsProductLevelLineImporter(CN8ToTbsProductLevelAbstractImporter):
    MAX_RECORDS_FOR_INSTANT_IMPORT = 10000
    NUMBER_EXPECTED_COLUMNS = 3

    existing_suggestions = None

    def __init__(self, session, **kwargs):
        ImportFileController.__init__(self, session, **kwargs)
        self.existing_suggestions = []

    def _import_row(self, row, controller, dryrun, tbs_id, regulation_period_id):
        cn = int(row[0])
        pl = row[1]
        is_b2b = None
        if len(row) > 2:
            is_b2b = row[2]

        if is_b2b in [None, '']:
            is_b2b = WeCN8PeriodProduct.B2B_NOT_APPLICABLE
        elif is_b2b.lower() == 'b2b':
            is_b2b = WeCN8PeriodProduct.IS_B2B_TRUE
        else:
            is_b2b = WeCN8PeriodProduct.IS_B2B_FALSE

        # get the cn8 else exception
        cn8 = self.sa_session.query(WeCN8).filter(WeCN8.code == cn).first()
        if cn8 is None:
            raise WeImportException(_("CN8 code %s is not in the system. "
                                      "Please add it before importing." % row[0]))

        # check for TBS
        tbs = self.sa_session.query(WeTbsLicense).filter(WeTbsLicense.id == tbs_id).first()

        if tbs is None:
            raise WeImportException(_('There is no TBS in the system with ID %s' % tbs_id))

        rp = self.sa_session.query(WeTbsRegulationPeriod).filter(
            and_(
                WeTbsRegulationPeriod.tbs == tbs,
                WeTbsRegulationPeriod.id == regulation_period_id
            )
        ).first()

        if rp is None:
            raise WeImportException(_('There is no Regulation Period with ID %d for this tbs %s' % (regulation_period_id, tbs.name)))

        # get existing local category else exception
        product_level = self.sa_session.query(WeTbsProductLevel).join(WeTbsRegulationPeriodProduct).filter(
            and_(
                WeTbsProductLevel.tbs == tbs,
                WeTbsProductLevel.label == unicode(pl),
                WeTbsRegulationPeriodProduct.tbs_regulation_period == rp
            )
        ).first()
        if product_level is None:
            raise WeImportException(_('TBS local category %s does not exist for this Regulation Period. '
                                      'Please add it before importing.' % pl))

        pp = [p for p in rp.period_products if p.tbs_product_level == product_level].pop()

        r = (cn8.code, product_level.label, is_b2b)
        if r in self.existing_suggestions:
            raise WeImportException(_("There are duplicate entries for CN8 %s. "
                                      "Make sure the CN8 is not linked "
                                      "to the same category twice or that B2B/B2C "
                                      "is different is applicable." % cn8.code))
        self.existing_suggestions.append(r)

        link = WeCN8PeriodProduct()
        link.cn8_code = cn8.code
        link.period_product = pp
        link.is_b2b = is_b2b
        self.sa_session.add(link)

        return cn8

    @staticmethod
    def get_notification_text():
        return ''

    @staticmethod
    def get_notification_subject():
        return 'CN8 to TBS local category import'

    @staticmethod
    def get_failure_notification_callback():
        return None

    @staticmethod
    def get_notification_callback():
        return None

class CN8ToTbsProductLevelCsvImporter(CN8ToTbsProductLevelLineImporter, Utf8CsvImporter):
    """
    Importer for CSV
    """

    def read_row(self, file_path):
        for row in self._read_row(file_path):
            yield self.get_only_expected_columns(row)


class CN8ToTbsProductLevelXlsxImporter(CN8ToTbsProductLevelLineImporter, XlsxImporter):
    """
    Importer for XLSX
    """

    def read_row(self, file_path):

        active_sheet = self._read_row(file_path)

        for row in active_sheet.iter_rows():
            values = []
            cell_i = 0
            row = self.get_only_expected_columns(row)
            for cell in row:
                cell_i += 1

                value = cell.internal_value

                if cell_i == 1 and value is not None:
                    if not isinstance(value, unicode):
                        WeImportException("Column 'Main category label' must be formatted to text.")
                if cell_i == 2:
                    if not isinstance(value, unicode):
                        WeImportException("Column 'Category label' must be formatted to text.")

                values.append(value)

            yield values